const through = require('through2').obj
const querystring = require('querystring')
const Authentic = require('authentic-client')
const AsyncCache = require('async-cache')

module.exports = Client

function Client (opts) {
  if (!(this instanceof Client)) return new Client(opts)
  const {email, password, apiHost, authServer} = opts
  if (!email) { throw new Error('Client could not be created - Email not provided') }
  if (!password) { throw new Error('Client could not be created - Password not provided') }
  this.apiHost = apiHost || 'https://adverse.lincx.la'
  const server = authServer || 'https://ix-id.lincx.la'

  this.requestInterval = opts.requestInterval || 10 * 1000

  this.client = Authentic({server})

  const cacheOpts = { email, password }
  this.cache = this.createCache(cacheOpts)
}

Client.prototype.createEventStream = function (params) {
  let totalResults = 0
  let type = (params['adId']) ? 'adId' : 'zoneId'
  let eventId = params['eventId'] || ''
  let stream = through()
  const self = this

  fetch(handler)

  function fetch (cb) {
    if (totalResults > 0) return
    const query = querystring.stringify({
      type,
      entityId: params[type],
      lastSyncId: eventId,
      limit: 1000
    })

    let url = `${self.apiHost}/sync?${query}`
    self.cache.get('client', function (err) {
      if (err) return cb(err)
      return self.client.get(url, cb)
    })
  }

  function handler (err, response) {
    if (err) return stream.emit('error', err)
    const result = response.result

    if (result && result.length <= 0) return setTimeout(fetch, self.requestInterval, handler)

    totalResults = result.length
    var maxId = createId(3600 * 1000)
    result.forEach(function (event) {
      if (event.eventId > maxId) return onFinish()

      if (event.eventId > eventId) eventId = event.eventId
      stream.write({event, onFinish})
    })
  }

  function onFinish (err) {
    if (err) return stream.emit('error', err)
    if (--totalResults === 0) return setTimeout(fetch, self.requestInterval, handler)
  }
  return stream
}

Client.prototype.createCache = function ({email, password}) {
  const self = this
  return new AsyncCache({
    maxAge: 30 * 24 * 3600 * 1000,
    load: function (key, cb) {
      self.client.login({email, password}, cb)
    }
  })
}

function createId (offset) {
  offset = offset || 0
  return 'c' + (new Date(Date.now() + offset).getTime()).toString(36)
}
